==> openconfig-models-17.4.0-sources.jar: org/opendaylight/yang/gen/v1/http/openconfig/net/yang/aaa/rev200730/aaa/authentication/top/AuthenticationBuilder.java <==
package org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top;
import java.lang.Class;
import java.lang.IllegalArgumentException;
import java.lang.NullPointerException;
import java.lang.Object;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import javax.annotation.processing.Generated;
import org.eclipse.jdt.annotation.NonNull;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.AaaAuthenticationAdminTop;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.AaaAuthenticationUserTop;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.admin.top.AdminUser;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.Config;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.State;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.user.top.Users;
import org.opendaylight.yangtools.yang.binding.AbstractAugmentable;
import org.opendaylight.yangtools.yang.binding.Augmentation;
import org.opendaylight.yangtools.yang.binding.CodeHelpers;
import org.opendaylight.yangtools.yang.binding.DataObject;

/**
 * Class that builds {@link Authentication} instances. Overall design of the class is that of a
 * <a href="https://en.wikipedia.org/wiki/Fluent_interface">fluent interface</a>, where method chaining is used.
 *
 * <p>
 * In general, this class is supposed to be used like this template:
 * <pre>
 *   <code>
 *     Authentication createAuthentication(int fooXyzzy, int barBaz) {
 *         return new AuthenticationBuilder()
 *             .setFoo(new FooBuilder().setXyzzy(fooXyzzy).build())
 *             .setBar(new BarBuilder().setBaz(barBaz).build())
 *             .build();
 *     }
 *   </code>
 * </pre>
 *
 * <p>
 * This pattern is supported by the immutable nature of Authentication, as instances can be freely passed around without
 * worrying about synchronization issues.
 *
 * <p>
 * As a side note: method chaining results in:
 * <ul>
 *   <li>very efficient Java bytecode, as the method invocation result, in this case the Builder reference, is
 *       on the stack, so further method invocations just need to fill method arguments for the next method
 *       invocation, which is terminated by {@link #build()}, which is then returned from the method</li>
 *   <li>better understanding by humans, as the scope of mutable state (the builder) is kept to a minimum and is
 *       very localized</li>
 *   <li>better optimization opportunities, as the object scope is minimized in terms of invocation (rather than
 *       method) stack, making <a href="https://en.wikipedia.org/wiki/Escape_analysis">escape analysis</a> a lot
 *       easier. Given enough compiler (JIT/AOT) prowess, the cost of th builder object can be completely
 *       eliminated</li>
 * </ul>
 *
 * @see Authentication
 *
 */
@Generated("mdsal-binding-generator")
public class AuthenticationBuilder {

    private AdminUser _adminUser;
    private Config _config;
    private State _state;
    private Users _users;


    Map<Class<? extends Augmentation<Authentication>>, Augmentation<Authentication>> augmentation = Map.of();

    /**
     * Construct an empty builder.
     */
    public AuthenticationBuilder() {
        // No-op
    }

    
    
    /**
     * Construct a new builder initialized from specified {@link AaaAuthenticationAdminTop}.
     *
     * @param arg AaaAuthenticationAdminTop from which the builder should be initialized
     */
    public AuthenticationBuilder(AaaAuthenticationAdminTop arg) {
        this._adminUser = arg.getAdminUser();
    }
    
    
    /**
     * Construct a new builder initialized from specified {@link AaaAuthenticationUserTop}.
     *
     * @param arg AaaAuthenticationUserTop from which the builder should be initialized
     */
    public AuthenticationBuilder(AaaAuthenticationUserTop arg) {
        this._users = arg.getUsers();
    }
    

    /**
     * Construct a builder initialized with state from specified {@link Authentication}.
     *
     * @param base Authentication from which the builder should be initialized
     */
    public AuthenticationBuilder(Authentication base) {
        final var aug = base.augmentations();
        if (!aug.isEmpty()) {
            this.augmentation = new HashMap<>(aug);
        }
        this._adminUser = base.getAdminUser();
        this._config = base.getConfig();
        this._state = base.getState();
        this._users = base.getUsers();
    }

    /**
     * Set fields from given grouping argument. Valid argument is instance of one of following types:
     * <ul>
     *   <li>{@link AaaAuthenticationAdminTop}</li>
     *   <li>{@link AaaAuthenticationUserTop}</li>
     * </ul>
     *
     * @param arg grouping object
     * @throws IllegalArgumentException if given argument is none of valid types or has property with incompatible value
    */
    public void fieldsFrom(DataObject arg) {
        boolean isValidArg = false;
        if (arg instanceof AaaAuthenticationAdminTop castArg) {
            this._adminUser = castArg.getAdminUser();
            isValidArg = true;
        }
        if (arg instanceof AaaAuthenticationUserTop castArg) {
            this._users = castArg.getUsers();
            isValidArg = true;
        }
        CodeHelpers.validValue(isValidArg, arg, "[AaaAuthenticationAdminTop, AaaAuthenticationUserTop]");
    }

    /**
     * Return current value associated with the property corresponding to {@link Authentication#getAdminUser()}.
     *
     * @return current value
     */
    public AdminUser getAdminUser() {
        return _adminUser;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getConfig()}.
     *
     * @return current value
     */
    public Config getConfig() {
        return _config;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getState()}.
     *
     * @return current value
     */
    public State getState() {
        return _state;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getUsers()}.
     *
     * @return current value
     */
    public Users getUsers() {
        return _users;
    }

    /**
     * Return the specified augmentation, if it is present in this builder.
     *
     * @param <E$$> augmentation type
     * @param augmentationType augmentation type class
     * @return Augmentation object from this builder, or {@code null} if not present
     * @throws NullPointerException if {@code augmentType} is {@code null}
     */
    @SuppressWarnings({ "unchecked", "checkstyle:methodTypeParameterName"})
    public <E$$ extends Augmentation<Authentication>> E$$ augmentation(Class<E$$> augmentationType) {
        return (E$$) augmentation.get(Objects.requireNonNull(augmentationType));
    }

    
    /**
     * Set the property corresponding to {@link Authentication#getAdminUser()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setAdminUser(final AdminUser value) {
        this._adminUser = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getConfig()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setConfig(final Config value) {
        this._config = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getState()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setState(final State value) {
        this._state = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getUsers()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setUsers(final Users value) {
        this._users = value;
        return this;
    }
    
    /**
      * Add an augmentation to this builder's product.
      *
      * @param augmentation augmentation to be added
      * @return this builder
      * @throws NullPointerException if {@code augmentation} is null
      */
    public AuthenticationBuilder addAugmentation(Augmentation<Authentication> augmentation) {
        if (!(this.augmentation instanceof HashMap)) {
            this.augmentation = new HashMap<>();
        }
    
        this.augmentation.put(augmentation.implementedInterface(), augmentation);
        return this;
    }
    
    /**
      * Remove an augmentation from this builder's product. If this builder does not track such an augmentation
      * type, this method does nothing.
      *
      * @param augmentationType augmentation type to be removed
      * @return this builder
      */
    public AuthenticationBuilder removeAugmentation(Class<? extends Augmentation<Authentication>> augmentationType) {
        if (this.augmentation instanceof HashMap) {
            this.augmentation.remove(augmentationType);
        }
        return this;
    }

    /**
     * A new {@link Authentication} instance.
     *
     * @return A new {@link Authentication} instance.
     */
    public @NonNull Authentication build() {
        return new AuthenticationImpl(this);
    }

    private static final class AuthenticationImpl
        extends AbstractAugmentable<Authentication>
        implements Authentication {
    
        private final AdminUser _adminUser;
        private final Config _config;
        private final State _state;
        private final Users _users;
    
        AuthenticationImpl(AuthenticationBuilder base) {
            super(base.augmentation);
            this._adminUser = base.getAdminUser();
            this._config = base.getConfig();
            this._state = base.getState();
            this._users = base.getUsers();
        }
    
        @Override
        public AdminUser getAdminUser() {
            return _adminUser;
        }
        
        @Override
        public Config getConfig() {
            return _config;
        }
        
        @Override
        public State getState() {
            return _state;
        }
        
        @Override
        public Users getUsers() {
            return _users;
        }
    
        private int hash = 0;
        private volatile boolean hashValid = false;
        
        @Override
        public int hashCode() {
            if (hashValid) {
                return hash;
            }
        
            final int result = Authentication.bindingHashCode(this);
            hash = result;
            hashValid = true;
            return result;
        }
    
        @Override
        public boolean equals(Object obj) {
            return Authentication.bindingEquals(this, obj);
        }
    
        @Override
        public String toString() {
            return Authentication.bindingToString(this);
        }
    }
}

==> openconfig-models-18.3.0-sources.jar: org/opendaylight/yang/gen/v1/http/openconfig/net/yang/aaa/rev200730/aaa/authentication/top/AuthenticationBuilder.java <==
package org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top;
import java.lang.Class;
import java.lang.IllegalArgumentException;
import java.lang.NullPointerException;
import java.lang.Object;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import javax.annotation.processing.Generated;
import org.eclipse.jdt.annotation.NonNull;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.AaaAuthenticationAdminTop;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.AaaAuthenticationUserTop;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.admin.top.AdminUser;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.Config;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.State;
import org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.user.top.Users;
import org.opendaylight.yangtools.yang.binding.AbstractAugmentable;
import org.opendaylight.yangtools.yang.binding.Augmentation;
import org.opendaylight.yangtools.yang.binding.CodeHelpers;
import org.opendaylight.yangtools.yang.binding.DataObject;

/**
 * Class that builds {@link Authentication} instances. Overall design of the class is that of a
 * <a href="https://en.wikipedia.org/wiki/Fluent_interface">fluent interface</a>, where method chaining is used.
 *
 * <p>
 * In general, this class is supposed to be used like this template:
 * <pre>
 *   <code>
 *     Authentication createAuthentication(int fooXyzzy, int barBaz) {
 *         return new AuthenticationBuilder()
 *             .setFoo(new FooBuilder().setXyzzy(fooXyzzy).build())
 *             .setBar(new BarBuilder().setBaz(barBaz).build())
 *             .build();
 *     }
 *   </code>
 * </pre>
 *
 * <p>
 * This pattern is supported by the immutable nature of Authentication, as instances can be freely passed around without
 * worrying about synchronization issues.
 *
 * <p>
 * As a side note: method chaining results in:
 * <ul>
 *   <li>very efficient Java bytecode, as the method invocation result, in this case the Builder reference, is
 *       on the stack, so further method invocations just need to fill method arguments for the next method
 *       invocation, which is terminated by {@link #build()}, which is then returned from the method</li>
 *   <li>better understanding by humans, as the scope of mutable state (the builder) is kept to a minimum and is
 *       very localized</li>
 *   <li>better optimization opportunities, as the object scope is minimized in terms of invocation (rather than
 *       method) stack, making <a href="https://en.wikipedia.org/wiki/Escape_analysis">escape analysis</a> a lot
 *       easier. Given enough compiler (JIT/AOT) prowess, the cost of th builder object can be completely
 *       eliminated</li>
 * </ul>
 *
 * @see Authentication
 *
 */
@Generated("mdsal-binding-generator")
public class AuthenticationBuilder {

    private AdminUser _adminUser;
    private Config _config;
    private State _state;
    private Users _users;


    Map<Class<? extends Augmentation<Authentication>>, Augmentation<Authentication>> augmentation = Map.of();

    /**
     * Construct an empty builder.
     */
    public AuthenticationBuilder() {
        // No-op
    }

    
    
    /**
     * Construct a new builder initialized from specified {@link AaaAuthenticationAdminTop}.
     *
     * @param arg AaaAuthenticationAdminTop from which the builder should be initialized
     */
    public AuthenticationBuilder(AaaAuthenticationAdminTop arg) {
        this._adminUser = arg.getAdminUser();
    }
    
    
    /**
     * Construct a new builder initialized from specified {@link AaaAuthenticationUserTop}.
     *
     * @param arg AaaAuthenticationUserTop from which the builder should be initialized
     */
    public AuthenticationBuilder(AaaAuthenticationUserTop arg) {
        this._users = arg.getUsers();
    }
    

    /**
     * Construct a builder initialized with state from specified {@link Authentication}.
     *
     * @param base Authentication from which the builder should be initialized
     */
    public AuthenticationBuilder(Authentication base) {
        final var aug = base.augmentations();
        if (!aug.isEmpty()) {
            this.augmentation = new HashMap<>(aug);
        }
        this._adminUser = base.getAdminUser();
        this._config = base.getConfig();
        this._state = base.getState();
        this._users = base.getUsers();
    }

    /**
     * Set fields from given grouping argument. Valid argument is instance of one of following types:
     * <ul>
     *   <li>{@link AaaAuthenticationAdminTop}</li>
     *   <li>{@link AaaAuthenticationUserTop}</li>
     * </ul>
     *
     * @param arg grouping object
     * @throws IllegalArgumentException if given argument is none of valid types or has property with incompatible value
    */
    public void fieldsFrom(DataObject arg) {
        boolean isValidArg = false;
        if (arg instanceof AaaAuthenticationAdminTop castArg) {
            this._adminUser = castArg.getAdminUser();
            isValidArg = true;
        }
        if (arg instanceof AaaAuthenticationUserTop castArg) {
            this._users = castArg.getUsers();
            isValidArg = true;
        }
        CodeHelpers.validValue(isValidArg, arg, "[AaaAuthenticationAdminTop, AaaAuthenticationUserTop]");
    }

    private static final class LazyEmpty {
        static final @NonNull Authentication INSTANCE = new AuthenticationBuilder().build();
    
        private LazyEmpty() {
            // Hidden on purpose
        }
    }
    
    /**
     * Get empty instance of Authentication.
     *
     * @return An empty {@link Authentication}
     */
    public static @NonNull Authentication empty() {
        return LazyEmpty.INSTANCE;
    }

    /**
     * Return current value associated with the property corresponding to {@link Authentication#getAdminUser()}.
     *
     * @return current value
     */
    public AdminUser getAdminUser() {
        return _adminUser;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getConfig()}.
     *
     * @return current value
     */
    public Config getConfig() {
        return _config;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getState()}.
     *
     * @return current value
     */
    public State getState() {
        return _state;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link Authentication#getUsers()}.
     *
     * @return current value
     */
    public Users getUsers() {
        return _users;
    }

    /**
     * Return the specified augmentation, if it is present in this builder.
     *
     * @param <E$$> augmentation type
     * @param augmentationType augmentation type class
     * @return Augmentation object from this builder, or {@code null} if not present
     * @throws NullPointerException if {@code augmentType} is {@code null}
     */
    @SuppressWarnings({ "unchecked", "checkstyle:methodTypeParameterName"})
    public <E$$ extends Augmentation<Authentication>> E$$ augmentation(Class<E$$> augmentationType) {
        return (E$$) augmentation.get(Objects.requireNonNull(augmentationType));
    }

    
    /**
     * Set the property corresponding to {@link Authentication#getAdminUser()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setAdminUser(final AdminUser value) {
        this._adminUser = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getConfig()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setConfig(final Config value) {
        this._config = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getState()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setState(final State value) {
        this._state = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link Authentication#getUsers()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AuthenticationBuilder setUsers(final Users value) {
        this._users = value;
        return this;
    }
    
    /**
      * Add an augmentation to this builder's product.
      *
      * @param augmentation augmentation to be added
      * @return this builder
      * @throws NullPointerException if {@code augmentation} is null
      */
    public AuthenticationBuilder addAugmentation(Augmentation<Authentication> augmentation) {
        if (!(this.augmentation instanceof HashMap)) {
            this.augmentation = new HashMap<>();
        }
    
        this.augmentation.put(augmentation.implementedInterface(), augmentation);
        return this;
    }
    
    /**
      * Remove an augmentation from this builder's product. If this builder does not track such an augmentation
      * type, this method does nothing.
      *
      * @param augmentationType augmentation type to be removed
      * @return this builder
      */
    public AuthenticationBuilder removeAugmentation(Class<? extends Augmentation<Authentication>> augmentationType) {
        if (this.augmentation instanceof HashMap) {
            this.augmentation.remove(augmentationType);
        }
        return this;
    }

    /**
     * A new {@link Authentication} instance.
     *
     * @return A new {@link Authentication} instance.
     */
    public @NonNull Authentication build() {
        return new AuthenticationImpl(this);
    }

    private static final class AuthenticationImpl
        extends AbstractAugmentable<Authentication>
        implements Authentication {
    
        private final AdminUser _adminUser;
        private final Config _config;
        private final State _state;
        private final Users _users;
    
        AuthenticationImpl(AuthenticationBuilder base) {
            super(base.augmentation);
            this._adminUser = base.getAdminUser();
            this._config = base.getConfig();
            this._state = base.getState();
            this._users = base.getUsers();
        }
    
        @Override
        public AdminUser getAdminUser() {
            return _adminUser;
        }
        
        @Override
        public Config getConfig() {
            return _config;
        }
        
        @Override
        public State getState() {
            return _state;
        }
        
        @Override
        public Users getUsers() {
            return _users;
        }
    
        @Override
        public AdminUser nonnullAdminUser() {
            return Objects.requireNonNullElse(getAdminUser(), org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.admin.top.AdminUserBuilder.empty());
        }
        
        @Override
        public Config nonnullConfig() {
            return Objects.requireNonNullElse(getConfig(), org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.ConfigBuilder.empty());
        }
        
        @Override
        public State nonnullState() {
            return Objects.requireNonNullElse(getState(), org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.top.authentication.StateBuilder.empty());
        }
        
        @Override
        public Users nonnullUsers() {
            return Objects.requireNonNullElse(getUsers(), org.opendaylight.yang.gen.v1.http.openconfig.net.yang.aaa.rev200730.aaa.authentication.user.top.UsersBuilder.empty());
        }
    
        private int hash = 0;
        private volatile boolean hashValid = false;
        
        @Override
        public int hashCode() {
            if (hashValid) {
                return hash;
            }
        
            final int result = Authentication.bindingHashCode(this);
            hash = result;
            hashValid = true;
            return result;
        }
    
        @Override
        public boolean equals(Object obj) {
            return Authentication.bindingEquals(this, obj);
        }
    
        @Override
        public String toString() {
            return Authentication.bindingToString(this);
        }
    }
}
